@@ -21,6 +21,7 @@
 #include <stddef.h>
 
 #include "data/format.h"
+#include "libpspp/compiler.h"
 #include "libpspp/float-format.h"
 #include "libpspp/integer-format.h"
 #include "libpspp/message.h"
@@ -86,9 +87,19 @@ void settings_set_blanks (double);
 int settings_get_max_messages (enum msg_severity);
 void settings_set_max_messages (enum msg_severity, int max);
 
+/* Macro settings. */
+bool settings_get_mexpand (void);
+void settings_set_mexpand (bool);
+
 bool settings_get_mprint (void);
 void settings_set_mprint (bool);
 
+int settings_get_miterate (void);
+void settings_set_miterate (int);
+
+int settings_get_mnest (void);
+void settings_set_mnest (int);
+
 int settings_get_mxloops (void);
 void settings_set_mxloops (int);
 
@@ -107,7 +118,7 @@ void settings_set_fuzzbits (int);
 
 /* Whether to show variable or value labels or the underlying value or variable
    name. */
-enum settings_value_show
+enum ATTRIBUTE ((packed)) settings_value_show
   {
     /* Use higher-level default.
        In a pivot_value, the default is taken from the pivot_table.
@@ -143,11 +154,12 @@ void unset_cmd_algorithm (void);
 enum fmt_type;
 bool settings_set_cc (const char *cc_string, enum fmt_type type);
 
-int settings_get_decimal_char (enum fmt_type type);
 void settings_set_decimal_char (char decimal);
 
+const struct fmt_settings *settings_get_fmt_settings (void);
 
-const struct fmt_number_style * settings_get_style (enum fmt_type type);
+double settings_get_small (void);
+void settings_set_small (double);
 
 char * settings_dollar_template (const struct fmt_spec *fmt);
 
@@ -175,4 +187,13 @@ void settings_set_output_routing (enum settings_output_type,
 enum settings_output_devices settings_get_output_routing (
   enum settings_output_type);
 
+enum settings_mdisplay
+  {
+    SETTINGS_MDISPLAY_TEXT,
+    SETTINGS_MDISPLAY_TABLES
+  };
+
+enum settings_mdisplay settings_get_mdisplay (void);
+void settings_set_mdisplay (enum settings_mdisplay);
+
 #endif /* !settings_h */
