@@ -85,10 +85,9 @@ void var_set_missing_values (struct variable *, const struct missing_values *);
 void var_clear_missing_values (struct variable *);
 bool var_has_missing_values (const struct variable *);
 
-bool var_is_value_missing (const struct variable *, const union value *,
-                           enum mv_class);
-bool var_is_num_missing (const struct variable *, double, enum mv_class);
-bool var_is_str_missing (const struct variable *, const uint8_t[], enum mv_class);
+enum mv_class var_is_value_missing (const struct variable *, const union value *);
+enum mv_class var_is_num_missing (const struct variable *, double);
+enum mv_class var_is_str_missing (const struct variable *, const uint8_t[]);
 
 /* Value labels. */
 const char *var_lookup_value_label (const struct variable *,
@@ -191,7 +190,7 @@ void var_set_leave (struct variable *, bool leave);
 bool var_must_leave (const struct variable *);
 
 /* Short names. */
-size_t var_get_short_name_cnt (const struct variable *);
+size_t var_get_n_short_names (const struct variable *);
 const char *var_get_short_name (const struct variable *, size_t idx);
 void var_set_short_name (struct variable *, size_t, const char *);
 void var_clear_short_names (struct variable *);
