@@ -16,13 +16,14 @@
 
 #include <config.h>
 
+#include "language/lexer/format-parser.h"
+
 #include <ctype.h>
 #include <stdint.h>
 #include <stdlib.h>
 
 #include "data/format.h"
 #include "data/variable.h"
-#include "language/lexer/format-parser.h"
 #include "language/lexer/lexer.h"
 #include "libpspp/message.h"
 #include "libpspp/misc.h"
@@ -34,7 +35,7 @@
 static bool
 parse_abstract_format_specifier__ (struct lexer *lexer,
                                    char type[FMT_TYPE_LEN_MAX + 1],
-                                   int *width, int *decimals)
+                                   uint16_t *width, uint8_t *decimals)
 {
   struct substring s;
   struct substring type_ss, width_ss, decimals_ss;
@@ -90,7 +91,7 @@ error:
 bool
 parse_abstract_format_specifier (struct lexer *lexer,
                                  char type[FMT_TYPE_LEN_MAX + 1],
-                                 int *width, int *decimals)
+                                 uint16_t *width, uint8_t *decimals)
 {
   bool ok = parse_abstract_format_specifier__ (lexer, type, width, decimals);
   if (ok)
@@ -98,10 +99,9 @@ parse_abstract_format_specifier (struct lexer *lexer,
   return ok;
 }
 
-/* Parses a format specifier from the token stream and returns
-   true only if successful.  Emits an error message on
-   failure.  The caller should call check_input_specifier() or
-   check_output_specifier() on the parsed format as
+/* Parses a format specifier from the token stream and returns true only if
+   successful.  Emits an error message on failure.  The caller should call
+   fmt_check_input() or fmt_check_output() on the parsed format as
    necessary.  */
 bool
 parse_format_specifier (struct lexer *lexer, struct fmt_spec *format)
