@@ -23,19 +23,16 @@
 
 #include "data/identifier.h"
 #include "data/variable.h"
+#include "language/lexer/segment.h"
 #include "libpspp/cast.h"
 #include "libpspp/compiler.h"
+#include "libpspp/message.h"
 #include "libpspp/prompt.h"
+#include "libpspp/str.h"
 
 struct lexer;
-
-/* The syntax mode for which a syntax file is intended. */
-enum lex_syntax_mode
-  {
-    LEX_SYNTAX_AUTO,            /* Try to guess intent. */
-    LEX_SYNTAX_INTERACTIVE,     /* Interactive mode. */
-    LEX_SYNTAX_BATCH            /* Batch mode. */
-  };
+struct lex_source;
+struct macro;
 
 /* Handling of errors. */
 enum lex_error_mode
@@ -52,7 +49,7 @@ enum lex_error_mode
 struct lex_reader
   {
     const struct lex_reader_class *class;
-    enum lex_syntax_mode syntax;
+    enum segmenter_mode syntax;
     enum lex_error_mode error;
     char *encoding;
     char *file_name;            /* NULL if not associated with a file. */
@@ -86,7 +83,7 @@ void lex_reader_set_file_name (struct lex_reader *, const char *file_name);
 /* Creating various kinds of lex_readers. */
 struct lex_reader *lex_reader_for_file (const char *file_name,
                                         const char *encoding,
-                                        enum lex_syntax_mode syntax,
+                                        enum segmenter_mode syntax,
                                         enum lex_error_mode error);
 struct lex_reader *lex_reader_for_string (const char *, const char *encoding);
 struct lex_reader *lex_reader_for_format (const char *, const char *, ...)
@@ -97,12 +94,16 @@ struct lex_reader *lex_reader_for_substring_nocopy (struct substring, const char
 struct lexer *lex_create (void);
 void lex_destroy (struct lexer *);
 
+/* Macros. */
+void lex_define_macro (struct lexer *, struct macro *);
+
 /* Files. */
 void lex_include (struct lexer *, struct lex_reader *);
 void lex_append (struct lexer *, struct lex_reader *);
 
 /* Advancing. */
 void lex_get (struct lexer *);
+void lex_get_n (struct lexer *, size_t n);
 
 /* Token testing functions. */
 bool lex_is_number (const struct lexer *);
@@ -123,13 +124,22 @@ bool lex_match (struct lexer *, enum token_type);
 bool lex_match_id (struct lexer *, const char *);
 bool lex_match_id_n (struct lexer *, const char *, size_t n);
 bool lex_match_int (struct lexer *, int);
+bool lex_at_phrase (struct lexer *, const char *s);
 bool lex_match_phrase (struct lexer *, const char *s);
 
 /* Forcible matching functions. */
 bool lex_force_match (struct lexer *, enum token_type) WARN_UNUSED_RESULT;
 bool lex_force_match_id (struct lexer *, const char *) WARN_UNUSED_RESULT;
 bool lex_force_int (struct lexer *) WARN_UNUSED_RESULT;
+bool lex_force_int_range (struct lexer *, const char *name,
+                          long min, long max) WARN_UNUSED_RESULT;
 bool lex_force_num (struct lexer *) WARN_UNUSED_RESULT;
+bool lex_force_num_range_closed (struct lexer *, const char *name,
+                                 double min, double max) WARN_UNUSED_RESULT;
+bool lex_force_num_range_halfopen (struct lexer *, const char *name,
+                                   double min, double max) WARN_UNUSED_RESULT;
+bool lex_force_num_range_open (struct lexer *, const char *name,
+                               double min, double max) WARN_UNUSED_RESULT;
 bool lex_force_id (struct lexer *) WARN_UNUSED_RESULT;
 bool lex_force_string (struct lexer *) WARN_UNUSED_RESULT;
 bool lex_force_string_or_id (struct lexer *) WARN_UNUSED_RESULT;
@@ -147,12 +157,20 @@ const char *lex_next_tokcstr (const struct lexer *, int n);
 double lex_next_tokval (const struct lexer *, int n);
 struct substring lex_next_tokss (const struct lexer *, int n);
 
+/* Looking at the current command, including lookahead and lookbehind. */
+int lex_ofs (const struct lexer *);
+const struct token *lex_ofs_token (const struct lexer *, int ofs);
+struct msg_location *lex_ofs_location (const struct lexer *, int ofs0, int ofs1);
+struct msg_point lex_ofs_start_point (const struct lexer *, int ofs);
+struct msg_point lex_ofs_end_point (const struct lexer *, int ofs);
+
+/* Token representation. */
+char *lex_next_representation (const struct lexer *, int n0, int n1);
+bool lex_next_is_from_macro (const struct lexer *, int n);
+
 /* Current position. */
-int lex_get_first_line_number (const struct lexer *, int n);
-int lex_get_last_line_number (const struct lexer *, int n);
-int lex_get_first_column (const struct lexer *, int n);
-int lex_get_last_column (const struct lexer *, int n);
 const char *lex_get_file_name (const struct lexer *);
+struct msg_location *lex_get_location (const struct lexer *, int n0, int n1);
 const char *lex_get_encoding (const struct lexer *);
 
 /* Issuing errors. */
@@ -161,9 +179,11 @@ void lex_next_error (struct lexer *, int n0, int n1, const char *, ...)
   PRINTF_FORMAT (4, 5);
 int lex_end_of_command (struct lexer *);
 
-void lex_error_expecting (struct lexer *, const char *, ...) SENTINEL(0);
+void lex_error_expecting (struct lexer *, ...) SENTINEL(0);
 #define lex_error_expecting(...) \
   lex_error_expecting(__VA_ARGS__, NULL_SENTINEL)
+void lex_error_expecting_valist (struct lexer *, va_list);
+void lex_error_expecting_array (struct lexer *, const char **, size_t n);
 
 void lex_sbc_only_once (const char *);
 void lex_sbc_missing (const char *);
@@ -180,10 +200,18 @@ void lex_next_error_valist (struct lexer *lexer, int n0, int n1,
   PRINTF_FORMAT (4, 0);
 
 /* Error handling. */
-enum lex_syntax_mode lex_get_syntax_mode (const struct lexer *);
+enum segmenter_mode lex_get_syntax_mode (const struct lexer *);
 enum lex_error_mode lex_get_error_mode (const struct lexer *);
 void lex_discard_rest_of_command (struct lexer *);
 void lex_interactive_reset (struct lexer *);
 void lex_discard_noninteractive (struct lexer *);
 
+/* Source code access. */
+void lex_set_message_handler (struct lexer *,
+                              void (*output_msg) (const struct msg *,
+                                                  struct lexer *));
+void lex_source_ref (const struct lex_source *);
+void lex_source_unref (struct lex_source *);
+struct substring lex_source_get_line (const struct lex_source *, int line);
+
 #endif /* lexer.h */
