@@ -134,7 +134,7 @@ median_execute (const struct dataset *ds,
 
       if (n_sample_test == false)
 	{
-	  struct val_node *vn = xzalloc (sizeof *vn);
+	  struct val_node *vn = XZALLOC (struct val_node);
 	  value_clone (&vn->val,  &nst->val1, var_get_width (nst->indep_var));
 	  hmap_insert (&map, &vn->node, value_hash (&nst->val1,
 					    var_get_width (nst->indep_var), 0));
@@ -159,7 +159,7 @@ median_execute (const struct dataset *ds,
 
 	  for (; (c = casereader_read (rr)) != NULL;)
 	    {
-	      if (var_is_value_missing (var, case_data (c, var), exclude))
+	      if (var_is_value_missing (var, case_data (c, var)) & exclude)
 		{
 		  case_unref (c);
 		  continue;
@@ -196,7 +196,7 @@ median_execute (const struct dataset *ds,
 	  const union value *val = case_data (c, var);
 	  const union value *indep_val = case_data (c, nst->indep_var);
 
-	  if (var_is_value_missing (var, case_data (c, var), exclude))
+	  if (var_is_value_missing (var, case_data (c, var)) & exclude)
 	    {
 	      continue;
 	    }
@@ -253,7 +253,7 @@ median_execute (const struct dataset *ds,
 	  }
 
 	results[v].n = count;
-	results[v].sorted_array = xcalloc (hmap_count (&map), sizeof (void*));
+	results[v].sorted_array = XCALLOC (hmap_count (&map), struct val_node *);
 	results[v].var = var;
 
 	HMAP_FOR_EACH (vn, struct val_node, node, &map)
