@@ -493,7 +493,7 @@ void
 llx_sort (struct llx *r0, struct llx *r1, llx_compare_func *compare, void *aux)
 {
   struct llx *pre_r0;
-  size_t output_run_cnt;
+  size_t output_run_len;
 
   if (r0 == r1 || llx_next (r0) == r1)
     return;
@@ -502,7 +502,7 @@ llx_sort (struct llx *r0, struct llx *r1, llx_compare_func *compare, void *aux)
   do
     {
       struct llx *a0 = llx_next (pre_r0);
-      for (output_run_cnt = 1; ; output_run_cnt++)
+      for (output_run_len = 1; ; output_run_len++)
         {
           struct llx *a1 = llx_find_run (a0, r1, compare, aux);
           struct llx *a2 = llx_find_run (a1, r1, compare, aux);
@@ -512,7 +512,7 @@ llx_sort (struct llx *r0, struct llx *r1, llx_compare_func *compare, void *aux)
           a0 = llx_merge (a0, a1, a1, a2, compare, aux);
         }
     }
-  while (output_run_cnt > 1);
+  while (output_run_len > 1);
 }
 
 /* Finds the extent of a run of nodes of increasing value
