@@ -61,7 +61,6 @@ acc (struct statistic *s, const struct ccase *cx,
 {
   struct box_whisker *bw = UP_CAST (s, struct box_whisker, parent.parent);
   bool extreme;
-  struct outlier *o;
 
   if (y > bw->hinges[2] + bw->step) /* Upper outlier */
     {
@@ -86,7 +85,7 @@ acc (struct statistic *s, const struct ccase *cx,
 
   /* y is an outlier */
 
-  o = xzalloc (sizeof *o) ;
+  struct outlier *o = XZALLOC (struct outlier);
   o->value = y;
   o->extreme = extreme;
   ds_init_empty (&o->label);
@@ -94,8 +93,9 @@ acc (struct statistic *s, const struct ccase *cx,
   if (bw->id_var)
     {
       char *s = data_out (case_data_idx (cx, bw->id_idx),
-                           var_get_encoding (bw->id_var),
-                           var_get_print_format (bw->id_var));
+                          var_get_encoding (bw->id_var),
+                          var_get_print_format (bw->id_var),
+                          settings_get_fmt_settings ());
 
       ds_put_cstr (&o->label, s);
       free (s);
@@ -104,7 +104,7 @@ acc (struct statistic *s, const struct ccase *cx,
     {
       ds_put_format (&o->label,
                      "%ld",
-                     (casenumber) case_data_idx (cx, bw->id_idx)->f);
+                     (casenumber) case_num_idx (cx, bw->id_idx));
     }
 
   ll_push_head (&bw->outliers, &o->ll);
@@ -144,7 +144,7 @@ struct box_whisker *
 box_whisker_create (const struct tukey_hinges *th,
 		    size_t id_idx, const struct variable *id_var)
 {
-  struct box_whisker *w = xzalloc (sizeof (*w));
+  struct box_whisker *w = XZALLOC (struct box_whisker);
   struct order_stats *os = &w->parent;
   struct statistic *stat = &os->parent;
 
