@@ -46,26 +46,6 @@
 
 #include <gettext.h>
 
-/* Returns a copy of IN with each underscore doubled.  The caller should free
-   the returned string (with free()) when it is no longer needed. */
-char *
-escape_underscores (const char *in)
-{
-  char *out = xmalloc (2 * strlen (in) + 1);
-  char *p;
-
-  p = out;
-  for (; *in != '\0'; in++)
-    {
-      if (*in == '_')
-        *p++ = '_';
-      *p++ = *in;
-    }
-  *p = '\0';
-
-  return out;
-}
-
 /* Formats a value according to VAR's print format and strips white space
    appropriately for VAR's type.  That is, if VAR is numeric, strips leading
    white space (because numbers are right-justified within their fields), and
@@ -95,7 +75,8 @@ value_to_text__ (union value v,
 {
   gchar *s;
 
-  s = data_out_stretchy (&v, encoding, format, NULL);
+  s = data_out_stretchy (&v, encoding, format, settings_get_fmt_settings (),
+                         NULL);
   if (fmt_is_numeric (format->type))
     g_strchug (s);
   else
@@ -137,25 +118,26 @@ text_to_value__ (const gchar *text,
 {
   int width = fmt_var_width (format);
 
-  if ( format->type != FMT_A)
+  if (format->type != FMT_A)
     {
-      if ( ! text ) return NULL;
+      if (! text) return NULL;
 
       {
 	const gchar *s = text;
 	while (*s)
 	  {
-	    if ( !isspace (*s))
+	    if (!isspace (*s))
 	      break;
 	    s++;
 	  }
 
-	if ( !*s) return NULL;
+	if (!*s) return NULL;
       }
     }
 
   value_init (val, width);
-  char *err = data_in (ss_cstr (text), UTF8, format->type, val, width, encoding);
+  char *err = data_in (ss_cstr (text), UTF8, format->type,
+                       settings_get_fmt_settings (), val, width, encoding);
 
   if (err)
     {
@@ -168,81 +150,9 @@ text_to_value__ (const gchar *text,
 }
 
 
-/* This function must be used whenever a filename generated by glib,
-   (eg, from gtk_file_chooser_get_filename) and passed to the C library,
-   (eg through a pspp syntax string).
-*/
-gchar *
-convert_glib_filename_to_system_filename (const gchar *fname, GError **err)
-{
-  gchar *output_name;
-
-#ifdef G_OS_WIN32
-  const gchar *target_encoding;
-  gchar *utf8_name = NULL;
-
-  g_get_charset (&target_encoding);
-
-  output_name = g_convert (fname, -1, target_encoding,
-			"UTF-8", NULL, NULL, err);
-#else
-  output_name = xstrdup (fname);
-#endif
-
-  return output_name;
-}
-
-
-
 #define _(msgid) gettext (msgid)
 #define N_(msgid) msgid
 
-
-static void
-give_help (void)
-{
-  GtkWidget *dialog;
-
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_MODAL,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_CLOSE,
-                                   _("Sorry. The help system hasn't yet "
-                                     "been implemented."));
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
-}
-
-void
-connect_help (GtkBuilder *xml)
-{
-  GSList *helps = gtk_builder_get_objects (xml);
-
-  GSList *i;
-  for ( i = helps; i ; i = g_slist_next (i))
-    {
-      GObject *o = i->data;
-      if ( GTK_IS_WIDGET (o) )
-	{
-	  const gchar *name = gtk_buildable_get_name (GTK_BUILDABLE (o));
-	  gchar s[12] = {0};
-
-	  if ( name)
-	    strncpy (s, name, 11);
-	  s[11] = '\0';
-
-
-	  if ( 0 == strcmp ("help_button", s))
-	    {
-	    g_signal_connect (o, "clicked", give_help, 0);
-	    }
-	}
-    }
-
-  g_slist_free (helps);
-}
-
-
 /* Create a deep copy of SRC */
 GtkListStore *
 clone_list_store (const GtkListStore *src)
@@ -253,10 +163,9 @@ clone_list_store (const GtkListStore *src)
   const gint n_cols =  gtk_tree_model_get_n_columns (GTK_TREE_MODEL (src));
   GType *types = g_malloc (sizeof (*types) *  n_cols);
 
-  int row = 0;
   GtkListStore *dest;
 
-  for (i = 0 ; i < n_cols; ++i )
+  for (i = 0 ; i < n_cols; ++i)
     types[i] = gtk_tree_model_get_column_type (GTK_TREE_MODEL (src), i);
 
   dest = gtk_list_store_newv (n_cols, types);
@@ -269,7 +178,7 @@ clone_list_store (const GtkListStore *src)
       GtkTreeIter dest_iter;
       gtk_list_store_append  (dest, &dest_iter);
 
-      for (i = 0 ; i < n_cols; ++i )
+      for (i = 0 ; i < n_cols; ++i)
 	{
 	  GValue val = {0};
 
@@ -278,7 +187,6 @@ clone_list_store (const GtkListStore *src)
 
 	  g_value_unset (&val);
 	}
-      row++;
     }
 
   g_free (types);
@@ -304,7 +212,7 @@ paste_syntax_to_window (gchar *syntax)
 
   GtkTextBuffer *buffer = NULL;
 
-  if ( NULL == the_syntax_pasteboard)
+  if (NULL == the_syntax_pasteboard)
     {
       the_syntax_pasteboard = psppire_syntax_window_new (NULL);
       g_signal_connect (the_syntax_pasteboard, "delete-event", G_CALLBACK (on_delete),
@@ -324,10 +232,21 @@ paste_syntax_to_window (gchar *syntax)
 }
 
 
-/* gtk_box_pack_start_defaults is deprecated.
-   Therefore we roll our own until a better solution is found */
-void
-psppire_box_pack_start_defaults (GtkBox *box, GtkWidget *widget)
+/* Return the width of an upper case M (in pixels) when rendered onto
+   WIDGET with its current style.  */
+gdouble
+width_of_m (GtkWidget *widget)
 {
-  gtk_box_pack_start (box, widget, TRUE, TRUE, 0);
+  PangoContext *context = gtk_widget_create_pango_context (widget);
+  PangoLayout *layout = pango_layout_new (context);
+  PangoRectangle rect;
+
+  pango_layout_set_text (layout, "M", 1);
+  pango_layout_get_extents (layout, NULL, &rect);
+
+  g_object_unref (G_OBJECT (layout));
+  g_object_unref (G_OBJECT (context));
+
+  return rect.width / (gdouble) PANGO_SCALE;
 }
+
