@@ -1,5 +1,6 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2004, 2005, 2006, 2010, 2011, 2012, 2013, 2014, 2015, 2016  Free Software Foundation
+   Copyright (C) 2004, 2005, 2006, 2010, 2011, 2012, 2013, 2014, 2015,
+   2016, 2020, 2021  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,13 +17,12 @@
 
 #include <config.h>
 
+#include "pre-initialisation.h"
+
 #include "ui/gui/psppire.h"
 
 #include <gtk/gtk.h>
 #include <stdlib.h>
-#if ENABLE_RELOCATABLE && defined(__APPLE__)
-#include <sys/stat.h>
-#endif
 
 #include "language/lexer/include-path.h"
 #include "libpspp/argv-parser.h"
@@ -30,6 +30,7 @@
 #include "libpspp/assertion.h"
 #include "libpspp/cast.h"
 #include "libpspp/copyleft.h"
+#include "libpspp/message.h"
 #include "libpspp/str.h"
 #include "libpspp/string-array.h"
 #include "libpspp/version.h"
@@ -37,6 +38,8 @@
 #include "ui/gui/psppire-syntax-window.h"
 #include "ui/gui/psppire-data-window.h"
 #include "ui/gui/psppire-output-window.h"
+#include "ui/gui/psppire-conf.h"
+#include "ui/gui/helper.h"
 
 #include "gl/configmake.h"
 #include "gl/progname.h"
@@ -93,10 +96,10 @@ init_dispatch (GSource * ss, GSourceFunc callback, gpointer user_data)
 }
 
 static GSourceFuncs init_funcs =
-  { init_prepare, init_check, init_dispatch, NULL, NULL, NULL };
+  {init_prepare, init_check, init_dispatch, NULL, NULL, NULL };
 
-GtkWidget *wsplash = 0;
-gint64 start_time = 0;
+static GtkWidget *wsplash = 0;
+static gint64 start_time = 0;
 
 
 static GtkWidget *
@@ -166,6 +169,120 @@ on_local_options (GApplication * application,
   return -1;
 }
 
+/* Use the imperitive mood for all entries in this table.
+   Each entry should end with a period.   */
+static const char *tips[] =
+  {
+#ifdef _WIN32
+   N_("PSPP runs best on free platforms such as GNU and GNU/Linux.  Windows is a non-free system.  As such, certain features might work sub-optimally.  For best results use a free system instead."),
+#endif
+   N_("Right click on variable lists to change between viewing the variables' names and their labels."),
+   N_("Click \"Paste\" instead of \"OK\" when running procedures.  This allows you to edit your commands before running them and you have better control over your work."),
+   N_("Directly import your spreadsheets using the \"File | Import Data\" menu."),
+   N_("For an easy way to convert string variables into numerically encoded variables, use \"Automatic Recode\"  which preserves the variable names as labels."),
+   N_("When browsing large data sets, use \"Windows | Split\" to see both ends of the data in the same view."),
+   N_("Export your reports to ODT format for easy editing with the Libreoffice.org suite."),
+   N_("Use \"Edit | Options\" to have your Output window automatically appear when statistics are generated."),
+   N_("To easily reorder your variables, drag and drop them in the Variable View or the Data View.")
+  };
+
+#define N_TIPS  (sizeof tips / sizeof tips[0])
+
+static void
+user_tip (GApplication *app)
+{
+  PsppireConf *conf = psppire_conf_new ();
+
+  gboolean show_tip = TRUE;
+  psppire_conf_get_boolean (conf, "startup", "show-user-tips", &show_tip);
+
+  if (!show_tip)
+    return;
+
+  GtkWindow *parent = gtk_application_get_active_window (GTK_APPLICATION (app));
+
+  GtkWidget *d =
+    gtk_dialog_new_with_buttons (_("Psppire User Hint"), parent,
+                                 GTK_DIALOG_MODAL,
+                                 GTK_MESSAGE_INFO,
+                                 0, 0,
+                                 NULL);
+
+  GtkWidget *pictogram = gtk_image_new_from_icon_name ("user-info", GTK_ICON_SIZE_DIALOG);
+
+  GtkWidget *next = gtk_button_new_with_mnemonic (_("_Next Tip"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (d), next, 1);
+
+  GtkWidget *close = gtk_button_new_with_mnemonic (_("_Close"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (d), close, GTK_RESPONSE_CLOSE);
+
+  gtk_window_set_transient_for (GTK_WINDOW (d), parent);
+
+  g_object_set (d,
+                "decorated", FALSE,
+                "skip-taskbar-hint", TRUE,
+                "skip-pager-hint", TRUE,
+                "application", app,
+                NULL);
+
+  GtkWidget *ca = gtk_dialog_get_content_area (GTK_DIALOG (d));
+
+  g_object_set (ca, "margin", 5, NULL);
+
+  GtkWidget *check = gtk_check_button_new_with_mnemonic ("_Show tips at startup");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), show_tip);
+
+  srand (time(0));
+  gint x = rand () % N_TIPS;
+  GtkWidget *label = gtk_label_new (gettext (tips[x]));
+
+  /* Make the font of the label a little larger than the other widgets.  */
+  {
+    GtkStyleContext *sc = gtk_widget_get_style_context (label);
+    GtkCssProvider *p = gtk_css_provider_new ();
+    const gchar *css = "* {font-size: 130%;}";
+    if (gtk_css_provider_load_from_data (p, css, strlen (css), NULL))
+      {
+        gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (p),
+                                        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+      }
+    g_object_unref (p);
+  }
+
+  /* It's more readable if the text is not all in one long line.  */
+  g_object_set (label, "wrap", TRUE, NULL);
+  gint width = PANGO_PIXELS (50.0 * width_of_m (label) * PANGO_SCALE);
+  gtk_window_set_default_size (GTK_WINDOW (d), width, -1);
+
+
+  if (pictogram)
+    gtk_box_pack_start (GTK_BOX (ca), pictogram, FALSE, FALSE, 5);
+  gtk_box_pack_start (GTK_BOX (ca), label, FALSE, FALSE, 5);
+  gtk_box_pack_end (GTK_BOX (ca), check, FALSE, FALSE, 5);
+
+  gtk_widget_show_all (d);
+
+  g_object_set (close,
+                "has-focus", TRUE,
+                "is-focus", TRUE,
+                NULL);
+
+  while (1 == gtk_dialog_run (GTK_DIALOG (d)))
+    {
+      if (++x >= N_TIPS) x = 0;
+      g_object_set (label, "label", gettext (tips[x]), NULL);
+    }
+
+  show_tip = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+  psppire_conf_set_boolean (conf,
+                            "startup", "show-user-tips",
+                            show_tip);
+
+  g_object_unref (conf);
+
+  gtk_widget_destroy (d);
+}
+
 
 static void
 on_startup (GApplication * app, gpointer ud)
@@ -177,6 +294,12 @@ on_startup (GApplication * app, gpointer ud)
       wsplash = create_splash_window ();
       gtk_application_add_window (GTK_APPLICATION (app),
                                   GTK_WINDOW (wsplash));
+
+      g_signal_connect_swapped (wsplash, "destroy", G_CALLBACK (user_tip), app);
+    }
+  else
+    {
+      g_signal_connect (app, "activate", G_CALLBACK (user_tip), NULL);
     }
 
   GMainLoop *loop = g_main_loop_new (context, FALSE);
@@ -236,15 +359,90 @@ wait_for_splash (GApplication *app, GtkWindow *x)
     }
 }
 
+static GtkWidget *fatal_error_dialog = NULL;
+static GtkWidget *fatal_error_label;
+static const char *diagnostic_info;
+
+static void
+fatal_error_handler (int sig)
+{
+  /* Reset SIG to its default handling so that if it happens again we won't
+     recurse. */
+  signal (sig, SIG_DFL);
+
+  static char message [1024];
+  strcpy (message, "proximate cause:    ");
+  switch (sig)
+    {
+    case SIGABRT:
+      strcat (message, "Assertion Failure/Abort");
+      break;
+    case SIGFPE:
+      strcat (message, "Floating Point Exception");
+      break;
+    case SIGSEGV:
+      strcat (message, "Segmentation Violation");
+      break;
+    default:
+      strcat (message, "Unknown");
+      break;
+    }
+  strcat (message, "\n");
+  strcat (message, diagnostic_info);
+
+  g_object_set (fatal_error_label,
+                "label", message,
+                NULL);
+
+  gtk_dialog_run (GTK_DIALOG (fatal_error_dialog));
+
+  /* Re-raise the signal so that we terminate with the correct status. */
+  raise (sig);
+}
+
 static void
 on_activate (GApplication * app, gpointer ud)
 {
+  struct sigaction fatal_error_action;
+  sigset_t sigset;
+  g_return_if_fail (0 == sigemptyset (&sigset));
+  fatal_error_dialog =
+    gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                            _("Psppire: Fatal Error"));
+
+  diagnostic_info = prepare_diagnostic_information ();
+
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (fatal_error_dialog),
+                                            _("You have discovered a bug in PSPP.  "
+                                              "Please report this to %s including all of the following information, "
+                                              "and a description of what you were doing when this happened."),
+                                            PACKAGE_BUGREPORT);
+
+  g_return_if_fail (fatal_error_dialog != NULL);
+
+  GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (fatal_error_dialog));
+  fatal_error_label = gtk_label_new ("");
+  g_object_set (fatal_error_label,
+                "selectable", TRUE,
+                "wrap", TRUE,
+                NULL);
+  gtk_container_add (GTK_CONTAINER (content_area), fatal_error_label);
+
+  gtk_widget_show_all (content_area);
+
+  fatal_error_action.sa_handler = fatal_error_handler;
+  fatal_error_action.sa_mask = sigset;
+  fatal_error_action.sa_flags = 0;
+
   post_initialise (app);
 
   GtkWindow *x = create_data_window ();
   gtk_application_add_window (GTK_APPLICATION (app), x);
 
   wait_for_splash (app, x);
+  sigaction (SIGABRT, &fatal_error_action, NULL);
+  sigaction (SIGSEGV, &fatal_error_action, NULL);
+  sigaction (SIGFPE,  &fatal_error_action, NULL);
 }
 
 static GtkWindow *
@@ -304,7 +502,7 @@ process_pre_start_arguments (int *argc, char ***argv)
   GOptionEntry oe[] = {
     {"version", 'V', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
      show_version_and_exit, N_("Show version information and exit"), 0},
-    {NULL}
+    {NULL, 0, 0, 0, NULL, "", 0}
   };
 
   GOptionContext *oc = g_option_context_new ("");
@@ -315,87 +513,11 @@ process_pre_start_arguments (int *argc, char ***argv)
   g_option_context_free (oc);
 }
 
-#if ENABLE_RELOCATABLE && defined(__APPLE__)
-static void
-pspp_macos_setenv (const char * progname)
-{
-  /* helper to set environment variables for pspp to be relocatable.
-   * Due to the latest changes it is not recommended to set it in the shell
-   * wrapper anymore.
-   */
-  gchar resolved_path[PATH_MAX];
-  /* on some OSX installations open file limit is 256 and GIMP needs more */
-  struct rlimit limit;
-  limit.rlim_cur = 10000;
-  limit.rlim_max = 10000;
-  setrlimit (RLIMIT_NOFILE, &limit);
-  if (realpath (progname, resolved_path))
-    {
-      gchar  tmp[PATH_MAX];
-      gchar *app_dir;
-      gchar  res_dir[PATH_MAX];
-      struct stat sb;
-
-      app_dir = g_path_get_dirname (resolved_path);
-      g_snprintf (tmp, sizeof(tmp), "%s/../../Resources", app_dir);
-      if (realpath (tmp, res_dir) && !stat (res_dir,&sb) && S_ISDIR (sb.st_mode))
-        g_print ("pspp is started as MacOS application\n");
-      else
-        return;
-      g_free (app_dir);
-
-      g_snprintf (tmp, sizeof(tmp), "%s/lib/gtk-3.0/3.0.0", res_dir);
-      g_setenv ("GTK_PATH", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/etc/gtk-3.0/gtk.immodules", res_dir);
-      g_setenv ("GTK_IM_MODULE_FILE", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/lib/gegl-0.4", res_dir);
-      g_setenv ("GEGL_PATH", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/lib/babl-0.1", res_dir);
-      g_setenv ("BABL_PATH", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/lib/gdk-pixbuf-2.0/2.10.0/loaders.cache", res_dir);
-      g_setenv ("GDK_PIXBUF_MODULE_FILE", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/etc/fonts", res_dir);
-      g_setenv ("FONTCONFIG_PATH", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/lib/gio/modules", res_dir);
-      g_setenv ("GIO_MODULE_DIR", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/etc/xdg", res_dir);
-      g_setenv ("XDG_CONFIG_DIRS", tmp, TRUE);
-      g_snprintf (tmp, sizeof(tmp), "%s/share", res_dir);
-      g_setenv ("XDG_DATA_DIRS", tmp, TRUE);
-
-      if (g_getenv ("HOME")!=NULL)
-        {
-          g_snprintf (tmp, sizeof(tmp),
-                      "%s/Library/Application Support/pspp/1.3/cache",
-                      g_getenv("HOME"));
-          g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
-        }
-    }
-}
-#endif
-
 int
 main (int argc, char *argv[])
 {
-
-#if ENABLE_RELOCATABLE && defined(__APPLE__)
-  /* remove MacOS session identifier from the command line args */
-  gint newargc = 0;
-  for (gint i = 0; i < argc; i++)
-    {
-      if (!g_str_has_prefix (argv[i], "-psn_"))
-        {
-          argv[newargc] = argv[i];
-          newargc++;
-        }
-    }
-  if (argc > newargc)
-    {
-      argv[newargc] = NULL; /* glib expects NULL terminated array */
-      argc = newargc;
-    }
-  pspp_macos_setenv (argv[0]);
-#endif
+  /* Some operating systems need to munge the arguments.  */
+  pre_initialisation (&argc, argv);
 
   set_program_name (argv[0]);
 
